@@ -1,6 +1,3 @@
-
-#' @importFrom htmltools tags tagList
-
 formats_1d <- c(
   "xx", "xx.", "xx.x", "xx.xx", "xx.xxx", "xx.xxxx",
   "xx%", "xx.%", "xx.x%", "xx.xx%", "xx.xxx%", "(N=xx)", ">999.9", ">999.99",
@@ -85,10 +82,7 @@ list_valid_aligns <- function() {
 #' @name check_formats
 #' @export
 is_valid_format <- function(x, stop_otherwise = FALSE) {
-  is_valid <- is.null(x) ||
-    (length(x) == 1 &&
-      (is.function(x) ||
-        x %in% unlist(list_valid_format_labels())))
+  is_valid <- is.null(x) || (length(x) == 1 && (is.function(x) || x %in% unlist(list_valid_format_labels())))
 
   if (stop_otherwise && !is_valid) {
     stop("format needs to be a format label, sprintf_format object, a function, or NULL")
@@ -107,12 +101,12 @@ is_valid_format <- function(x, stop_otherwise = FALSE) {
 #' @name check_formats
 #' @export
 check_aligns <- function(algn) {
-    if(anyNA(algn))
-        stop("Got missing-value for text alignment.")
-    invalid <- setdiff(algn, list_valid_aligns())
-    if(length(invalid) > 0) {
-        stop("Unsupported text-alignment(s): ",
-             paste(invalid, collapse = ", "))
+  if (anyNA(algn)) {
+    stop("Got missing-value for text alignment.")
+  }
+  invalid <- setdiff(algn, list_valid_aligns())
+  if (length(invalid) > 0) {
+    stop("Unsupported text-alignment(s): ", paste(invalid, collapse = ", "))
   }
   invisible(TRUE)
 }
@@ -432,9 +426,12 @@ format_value <- function(x, format = NULL, output = c("ascii", "html"), na_str =
 setClassUnion("FormatSpec", c("NULL", "character", "function", "list"))
 setClassUnion("characterOrNULL", c("NULL", "character"))
 setClass("fmt_config",
-         slots = c(format = "FormatSpec",
-                   format_na_str = "characterOrNULL",
-                   align = "characterOrNULL"))
+  slots = c(
+    format = "FormatSpec",
+    format_na_str = "characterOrNULL",
+    align = "characterOrNULL"
+  )
+)
 
 #' Format Configuration
 #'
