@@ -5,8 +5,6 @@
 #' `%||%` If length-0 alternative operator
 #' @name ifnotlen0
 #'
-#'
-#'
 #' @param a ANY. Element to select only if it is not length 0
 #' @param b ANY. Element to select if \code{a} is length 0
 #' @export
@@ -18,4 +16,10 @@
 #' NULL %||% "hi"
 #' @return `a`, unless it  is length 0, in which case  `b` (even in the
 #'     case `b` is also length 0)
-`%||%` <- function(a, b) if (length(a) == 0) b else a
+`%||%` <- function(a, b) {
+  if (length(a) == 0) {
+    b
+  } else {
+    a
+  }
+}
